// RotateBitmap.h | by Doug Manley
////////////////////////////////////////////////////////
#ifndef ROTATEBITMAP_H
#define ROTATEBITMAP_H
////////////////////////////////////////////////////////
#include <math.h>
////////////////////////////////////////////////////////
#ifndef _WINDOWS_
template<class T>
const T& min(const T& x, const T& y) {
	if( x < y ) {
		return(x);
	}
	return(y);
}
template<class T>
const T& max(const T& x, const T& y) {
	if( x > y ) {
		return(x);
	}
	return(y);
}
#endif
////////////////////////////////////////////////////////
/* This sucker rotates a bitmap (s) a number of degrees.
   These are in Polar? format, with 0 being North and
	90 being East.
	It returns the new bitmap on success and NULL if it
	was passed bad input.
*/
const double pi = 3.1415926535;
//const double pi = 3.141592653589793433832795;
const double piOver180 = pi/(double)180;
SDL_Surface* RotateBitmap( SDL_Surface *s, double angleInDegrees ) {
	if( !s || s == NULL ) {
		return( NULL );
	}

	int min_x = 0, max_x = s->w;
	int min_y = 0, max_y = s->h;


	double radians = angleInDegrees * piOver180;

	float cosine = (float)cos(radians);
	float sine = (float)sin(radians);

		
	// Compute dimensions of the resulting bitmap
	// First get the coordinates of the 3 corners other than origin
	int x1 = (int)(-(float)s->h * sine);
	int y1 = (int)( (float)s->h * cosine);
	int x2 = (int)( (float)s->w * cosine - (float)s->h * sine);
	int y2 = (int)( (float)s->h * cosine + (float)s->w * sine);
	int x3 = (int)( (float)s->w * cosine);
	int y3 = (int)( (float)s->w * sine);//*/


	/*double cosine = cos(radians);
	double sine = sin(radians);

	// Compute dimensions of the resulting bitmap
	// First get the coordinates of the 3 corners other than origin
	int x1 = (int)(-(double)s->h * sine);
	int y1 = (int)( (double)s->h * cosine);
	int x2 = (int)( (double)s->w * cosine - (double)s->h * sine);
	int y2 = (int)( (double)s->h * cosine + (double)s->w * sine);
	int x3 = (int)( (double)s->w * cosine);
	int y3 = (int)( (double)s->w * sine);//*/

	int minx = min(0,min(x1, min(x2,x3)));
	int miny = min(0,min(y1, min(y2,y3)));
	int maxx = max(0,max(x1, max(x2,x3)));
	int maxy = max(0,max(y1, max(y2,y3)));


	//printf("angle: %i | min: (%i,%i) | max (%i,%i)\n",i, minx,miny, maxx,maxy);

	int w = maxx - minx;
	int h = maxy - miny;

	SDL_Surface *spA = SDL_CreateRGBSurface( s->flags&SDL_HWSURFACE?SDL_HWSURFACE:SDL_SWSURFACE,w,h,s->format->BitsPerPixel, s->format->Rmask,s->format->Gmask,s->format->Bmask,s->format->Amask );
	if( s->flags&SDL_SRCCOLORKEY ) {
		SDL_FillRect( spA,NULL,s->format->colorkey );
		Uint32 rle = (s->flags&SDL_RLEACCEL)?SDL_RLEACCEL:0;
		SDL_SetColorKey( spA, SDL_SRCCOLORKEY|rle, s->format->colorkey );
	}
	SDL_Surface *sp = SDL_DisplayFormat(spA);
		SDL_FreeSurface(spA);



	// Now do the actual rotating - a pixel at a time
	// Computing the destination point for each source point
	// will leave a few pixels that do not get covered
	// So we use a reverse transform - e.i. compute the source point
	// for each destination point
	bool mustLocks = (SDL_MUSTLOCK(s)==0?false:true);
	if( mustLocks ) {  //printf("s must be locked... locking...\n");
		SDL_LockSurface( s );
	}
	bool mustLocksp = (SDL_MUSTLOCK(sp)==0?false:true);
	if( mustLocksp ) {  //printf("sp must be locked... locking...\n");
		SDL_LockSurface( sp );
	}
	for( int y = miny; y < maxy; y++ ) {
		for( int x = minx; x < maxx; x++ ) {
			int sourcex = (int)(x*cosine + y*sine);
			int sourcey = (int)(y*cosine - x*sine);
			if( sourcex >= 0 && sourcex < s->w && sourcey >= 0  && sourcey < s->h ) {
				putpixel(sp, x-minx,y-miny, getpixel(s,sourcex,sourcey));
			}
		}
	}
	if( mustLocks ) {  SDL_UnlockSurface( s );  }
	if( mustLocksp ) {  SDL_UnlockSurface( sp );  }

	return( sp );
}
////////////////////////////////////////////////////////
////////////////////////////////////////////////////////
#endif
////////////////////////////////////////////////////////